#version 330
// Resolve gbuffer

#pragma fragment_shader

in vec2 coord;
out vec4 fragment;

uniform sampler2D bufferD;
uniform sampler2D buffer0;
uniform sampler2D buffer1;

uniform vec3 lightDirection;
uniform vec3 lightColour;

uniform sampler2D shadow;
uniform mat4 shadowMatrix;
uniform mat4 inverseProjection;
uniform mat4 inverseView;


vec3 getPosition(vec2 coord) {
    float rawDepth = texture(bufferD, coord).r * 2.f - 1.f;
    vec4 pos = vec4(inverseProjection * vec4(coord * 2.f - 1.f, rawDepth, 1.f));
	return (inverseView * (pos / pos.w)).xyz;
}

void main() {
	float depth = texture(bufferD, coord.xy).x;
	vec4  buf0 = texture(buffer0, coord.xy);
	vec4  buf1 = texture(buffer1, coord.xy);

	if(depth == 1.0) discard;
	vec3 albedo = buf0.rgb;
	vec3 normal = buf1.rgb * 2.0 - 1.0;
	float rough = buf0.a;


	// Basic direcional light
	float l = dot( normalize(normal), normalize(lightDirection));
	float s = (l+1.0)/1.3*0.2+0.1; // dark side lighting
	float light = max(s, l);

	// Main shadow
	vec3 position = getPosition(coord);
	vec4 shadowCoord = shadowMatrix * vec4(position, 1.0);
	vec4 shadowValue = texture(shadow, clamp(shadowCoord.xy * 0.5 + 0.5, 0.01, 0.99));
	vec2 shadowEdge = smoothstep(0.9, 1.0, abs(shadowCoord.xy));
	float shade = step(shadowCoord.z / shadowCoord.w * 0.5 + 0.5, shadowValue.r + 0.003);
	shade = mix(shade, 1.0, shadowEdge.x * shadowEdge.y);
	light *= shade * 0.4 + 0.6;


	fragment = vec4(albedo * light * lightColour, 1.0);

	// Glow
	fragment.rgb *= 1.0 + buf1.a * 8.0;



	//fragment = vec4(albedo * ambient, 1);
	gl_FragDepth = depth;
}


